#!../../../scripts/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: test_suite.sh,v $
#	$Revision: 1.14 $
#	$Date: 2006/11/01 09:56:46 $
#	$Author: zapreevis $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script runs tests
#

#This function allows for terminating the MRMC performance test case execution by the timeout
#@param ${1} is the timeout value
#@param ${2} is the MRMC process ID, should be set to -1 if MRMC has already terminated
#@param ${3} is the file where we will
#write the timeout line in case of killing the test
doWait () {
	let "kill_indx=${1}+1"
	
	echo -n "0"
	BSP="\b"
	#Test for the case when MRMC has already terminated
	if test "${2}" != "-1"; then
		for (( i=1 ; i <= ${1} ; i++ )) ; do
			sleep 1
			CHECK_MRMC=`eval ps ${2} | grep ${2}`
			if test ! -z "${CHECK_MRMC}"; then
				if test "${i}" -le "10"; then
					BSP="\b";
				elif test "${i}" -le "100"; then
					BSP="\b\b"
				elif test "${i}" -le "1000"; then
					BSP="\b\b\b"
				elif test "${i}" -le "10000"; then
					BSP="\b\b\b\b"
				elif test "${i}" -le "100000"; then
					BSP="\b\b\b\b\b"
				elif test "${i}" -le "1000000"; then
					BSP="\b\b\b\b\b\b"
				elif test "${i}" -le "10000000"; then
					#It is about 115 days, I doubt anyone will need more :)
					BSP="\b\b\b\b\b\b\b"
				fi
				echo -ne "${BSP}${i}"
			else
				let "i=${1}+2"
			fi
		done
	fi

	if test "${i}" -eq "${kill_indx}"; then
		kill -9 ${2}
		#WARNING: sometimes one run of MRMC produces several "time to compute:"
		#statesments, for example when we do formula independent lumping
		#Thus we print two timeout messages in order to make scripts work.
		#We do not expect this to influence the average time for the testcase run
		#since if one testcase run takes more then ${1} time then others most
		#probably will be the same.
		let "TIME_OUT_MILLISEC=${1}*1000000"
		echo "Timeout 1: ${TIME_OUT_MILLISEC} micro sec(s)" >> ${3}
		echo "Timeout 2: ${TIME_OUT_MILLISEC} micro sec(s)" >> ${3}
		echo -ne "${BSP}-"
	else
		echo -ne "${BSP}+"
	fi
}

TESTS=`eval cat ${1} | sed -e '/#.*/d'`

INPUTS=`eval cat ${2} | sed -e '/#.*/d'`

##########################################
#%%%%%%%%%%%%EXECUTE TESTS%%%%%%%%%%%%%%%#
##########################################

export PATH=.:${PATH}		#Add local directory to the path, otherwise the tests will not run

############################################
#Perform tests, with all given input files,#
#   for the given number of repetitions,   #
#         with all given options.          #
############################################

#Run
for test in ${TESTS}
do
	#Set TEST_NAME
	export TEST_NAME=${test}
	#Set the input files for the given test
	export INPUT_DATA_FILES=`eval source input.data.files`
	
	cd ${test}
	echo "    ${test}: "
	
	#Generate the model and initialize
	echo -n "      Generating the model .........."
	rm -f ${TEST_NAME}.prism.log.out
	${TEST_NAME}.sh 1>${TEST_NAME}.prism.log.out
	echo "DONE"
	
	########################################################
	#    Do functional part of the performance testing,    #
	#just to make sure that MRMC still functions correctly.#
	########################################################

	#Run PERFORMANCE TESTS in the functional mode first
	#In order to test that MRMC works fine
	echo "        Functional test:"
	for input in ${INPUTS}
	do
		input_file=${input}.input
		echo "          ${input}:"
		#Run MRMC with different options
		cat ../option_list | sed -e '/#.*/d' | while read options; do
			#Used for constructing the test case name
			OPTS=`eval echo ${options} | sed -e 's/ /./'`
			TEST_CASE_NAME=${TEST_NAME}.${OPTS}.${input}
			
			echo -n "            ${TEST_CASE_NAME}: "
			
			#Run the test
			${MRMC} ${options} ${INPUT_DATA_FILES} < ../${input_file} 2>&1 > ${TEST_CASE_NAME}.out &
			#Collect the PID of the process
			PID=$!
			#Do the waiting with the timeout
			doWait ${PERFORMANCE_TEST_TIMEOUT_SECS} ${PID} ${TEST_CASE_NAME}.out
			
			#cat "" >> ${TEST_CASE_NAME}.golden
			#Run the functional testing like script to tests for PASS/FAIL
			${LUMPING_SHELL_SCRIPTS}/pf.sh ${TEST_CASE_NAME}
		done
	done

	#Rerun PERFORMANCE TESTS with different input files
	echo "        Performance test:"
	for input in ${INPUTS}
	do
		input_file=${input}.input
		echo -n "          ${input}:"
		#Rerun the experiments the required number of times
		for (( i=0 ; i < ${NUMBER_OF_PERFORMANCE_REPETITIONS} ; i=i+1 )) ; do
			echo -n " ${i}:"
			#Run MRMC with different options
			cat ../option_list | sed -e '/#.*/d' | while read options; do
				OPTS=`eval echo ${options} | sed -e 's/ /./'`
				${MRMC} ${options} ${INPUT_DATA_FILES} < ../${input_file} | grep compute >> ${TEST_NAME}.${OPTS}.${input}.results &
				#Collect the PID of the process
				#WARNING: This is not good because we use the name
				#which can be changed i.e. "mrmc" and also because
				#using the following line we collect all instance
				#of mrmc running on the machine
				PID=`eval pidof mrmc`
				#The MRMC could have finished working by now!
				if test -z "${PID}"; then
					PID="-1"
				fi
				#Do the waiting with the timeout
				doWait ${PERFORMANCE_TEST_TIMEOUT_SECS} ${PID} ${TEST_NAME}.${OPTS}.${input}.results
			done
		done
		echo " DONE"
	done
	
	#Remove the tra files and deinitialize
	rm -f  ${INPUT_DATA_FILES}
	#Move to the test suite directory
	cd ..
done

#Perform data collection and processing for the statistics.
if test ${NUMBER_OF_PERFORMANCE_REPETITIONS} -ne  0; then
	${LUMPING_SHELL_SCRIPTS}/statistics.sh ${1} ${2}
fi
